#!/usr/bin/perl -T

use lib '.'; use lib 't';
use SATest; sa_t_init("spam");

use Test::More tests => 7;

# ---------------------------------------------------------------------------

%patterns = (

q{ Subject: There yours for FREE!}, 'subj',
q{ X-Spam-Status: Yes, score=}, 'status',
q{ X-Spam-Flag: YES}, 'flag',
q{ X-Spam-Level: **********}, 'stars',
q{ TEST_ENDSNUMS }, 'endsinnums',
q{ TEST_NOREALNAME }, 'noreal',

);

ok (sarun ("-L -t < data/spam/001", \&patterns_run_cb));
ok_all_patterns();
